#!/usr/bin/env python

# Python 2/3 compatibility.
from __future__ import print_function

import os
import signal
import subprocess
import sys
import tempfile
import textwrap

from pkg_resources import load_entry_point


class SigException(BaseException):
    def __init__(self, signum):
        super(SigException, self).__init__()
        self.signum = signum

def xsession_sighandler(signum, frame):
    raise SigException(signum)

def xserver_start(display, executable='Xvfb', authfile=None):
    pid = os.fork()
    if pid != 0:
        return pid
    if authfile is None:
        authfile = os.devnull
    # This will make the xserver send us a SIGUSR1 when ready.
    signal.signal(signal.SIGUSR1, signal.SIG_IGN)
    cmd = [
        executable,
        '-auth', authfile,
        '-noreset',
        display,
    ]
    print('starting xserver: `{0}`'.format(' '.join(cmd)))
    os.execlp(cmd[0], *cmd)

def tests_run(display, authfile=None):
    pid = os.fork()
    if pid != 0:
        return pid
    if authfile is None:
        authfile = os.devnull
    os.environ['DISPLAY'] = display
    os.environ['XAUTHORITY'] = authfile
    cmd = [
        sys.executable,
        '-c', textwrap.dedent(
            '''
            from pkg_resources import load_entry_point
            sys.exit(load_entry_point(
                'nose', 'console_scripts', 'nosetests',
            )())
            '''
        ).lstrip(),
        '--exe', '--with-xunit', '--verbosity=3',
    ]
    has_custom_tests = False
    for arg in sys.argv[1:]:
        if not arg.startswith('-'):
            has_custom_tests = True
        cmd.append(arg)
    if not has_custom_tests:
        cmd.extend(('test/', 'examples/run_examples.py'))
    print('running tests: `{0}`'.format(' '.join(cmd)))
    sys.argv = cmd
    try:
        load_entry_point('nose', 'console_scripts', 'nosetests')()
    except SystemExit as err:
        code = err.code
    else:
        code = 0
    os._exit(code)


def runtests():

    cleanup_funcs = []

    try:
        if hasattr(sys, 'pypy_version_info'):
            server_display = ':8'
        else:
            server_display = ':9'
        server_display += ''.join(str(n) for n in sys.version_info[:3])

        # Setup a temporary authentication file.
        cookie = subprocess.check_output('mcookie').strip()
        authfile = tempfile.NamedTemporaryFile(delete=False)
        cleanup_funcs.append(lambda: os.unlink(authfile.name))
        authfile.close()
        subprocess.check_call((
            'xauth',
            '-f', authfile.name,
            'add', server_display, '.', cookie,
        ))

        # Setup signal handler to wait for xserver to be ready.
        signal.signal(signal.SIGUSR1, xsession_sighandler)

        # Start xserver.
        server_pid = xserver_start(server_display, authfile=authfile.name)
        cleanup_funcs.append(lambda: os.waitpid(server_pid, 0))
        cleanup_funcs.append(lambda: os.kill(server_pid, signal.SIGTERM))

        # Give the server 3 seconds to start.
        signal.alarm(3)

        # Wait for server to be ready.
        try:
            signal.pause()
        except SigException as err:
            assert signal.SIGUSR1 == err.signum
            signal.alarm(0)

        # Run tests.
        tests_pid = tests_run(server_display, authfile=authfile.name)
        pid, status = os.waitpid(tests_pid, 0)
        assert pid == tests_pid
        sys.exit(status >> 8)

    except KeyboardInterrupt:
        sys.exit(1)

    finally:
        for func in reversed(cleanup_funcs):
            func()


if __name__ == '__main__':
    runtests()



#!/usr/bin/python
#
# examples/xinput.py -- demonstrate the XInput extension
#
#    Copyright (C) 2012 Outpost Embedded, LLC
#      Forest Bond <forest.bond@rapidrollout.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
#    Free Software Foundation, Inc.,
#    59 Temple Place,
#    Suite 330,
#    Boston, MA 02111-1307 USA

# Python 2/3 compatibility.
from __future__ import print_function

import sys
import os

# Change path so we find Xlib
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from Xlib.display import Display
from Xlib.ext import xinput


def print_hierarchy_changed_event(event):
    print('<deviceid=%s time=%s flags=%s info={' % (
        event.data.deviceid,
        event.data.time,
        event.data.flags,
        ))
    for info in event.data.info:
        print_info(info)
    print('}>')


def print_info(info):
    print('  <deviceid=%s attachment=%s type=%s enabled=%s flags=%s>' % (
        info.deviceid,
        info.attachment,
        info.type,
        info.enabled,
        info.flags,
        ))


def main(argv):
    display = Display()
    try:
        extension_info = display.query_extension('XInputExtension')
        xinput_major = extension_info.major_opcode

        version_info = display.xinput_query_version()
        print('Found XInput version %u.%u' % (
          version_info.major_version,
          version_info.minor_version,
        ))

        screen = display.screen()
        screen.root.xinput_select_events([
          (xinput.AllDevices, xinput.HierarchyChangedMask),
        ])

        while True:
            event = display.next_event()
            if (
              event.type == display.extension_event.GenericEvent
              and event.extension == xinput_major
              and event.evtype == 11
            ):
                print_hierarchy_changed_event(event)

    finally:
        display.close()


if __name__ == '__main__':
    sys.exit(main(sys.argv))



#!/usr/bin/env python
#
# examples/get_selection.py -- demonstrate getting selections
# (equivalent to pasting from the clipboard)
#
#	Copyright (C) 2013 Peter Liljenberg <peter.liljenberg@gmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
#    Free Software Foundation, Inc.,
#    59 Temple Place,
#    Suite 330,
#    Boston, MA 02111-1307 USA

import binascii
import sys
import os

# Change path so we find Xlib
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from Xlib import X, display, Xutil, Xatom


def log(msg, *args):
    sys.stderr.write(msg.format(*args) + '\n')

def error(msg, *args):
    log(msg, *args)
    sys.exit(1)


def main():
    if len(sys.argv) < 2 or len(sys.argv) > 3:
        sys.exit('usage: {0} SELECTION [TYPE]\n\n'
                 'SELECTION is typically PRIMARY, SECONDARY or CLIPBOARD.\n'
                 'If TYPE is omitted, the available types for the selction are listed.'
                 .format(sys.argv[0]))

    d = display.Display()

    sel_name = sys.argv[1]
    sel_atom = d.get_atom(sel_name)

    if len(sys.argv) > 2:
        target_name = sys.argv[2]
        target_atom = d.get_atom(target_name)
    else:
        target_name = 'TARGETS'
        target_atom = d.get_atom('TARGETS')

    # Ask the server who owns this selection, if any
    owner = d.get_selection_owner(sel_atom)

    if owner == X.NONE:
        log('No owner for selection {0}', sel_name)
        return

    log('selection {0} owner: 0x{1:08x} {2}',
        sel_name, owner.id, owner.get_wm_name())

    # Create ourselves a window and a property for the returned data
    w = d.screen().root.create_window(
        0, 0, 10, 10, 0, X.CopyFromParent)
    w.set_wm_name(os.path.basename(sys.argv[0]))

    data_atom = d.get_atom('SEL_DATA')

    # The data_atom should not be set according to ICCCM, and since
    # this is a new window that is already the case here.

    # Ask for the selection.  We shouldn't use X.CurrentTime, but
    # since we don't have an event here we have to.
    w.convert_selection(sel_atom, target_atom, data_atom, X.CurrentTime)

    # Wait for the notification that we got the selection
    while True:
        e = d.next_event()
        if e.type == X.SelectionNotify:
            break

    # Do some sanity checks
    if (e.requestor != w
        or e.selection != sel_atom
        or e.target != target_atom):
        error('SelectionNotify event does not match our request: {0}', e)

    if e.property == X.NONE:
        log('selection lost or conversion to {0} failed',
            target_name)
        return

    if e.property != data_atom:
        error('SelectionNotify event does not match our request: {0}', e)

    # Get the data
    r = w.get_full_property(data_atom, X.AnyPropertyType,
                            sizehint = 10000)

    # Can the data be used directly or read incrementally
    if r.property_type == d.get_atom('INCR'):
        log('reading data incrementally: at least {0} bytes', r.value[0])
        handle_incr(d, w, data_atom, target_name)
    else:
        output_data(d, r, target_name)

    # Tell selection owner that we're done
    w.delete_property(data_atom)


def handle_incr(d, w, data_atom, target_name):
    # This works by us removing the data property, the selection owner
    # getting a notification of that, and then setting the property
    # again with more data.  To notice that, we must listen for
    # PropertyNotify events.
    w.change_attributes(event_mask = X.PropertyChangeMask)

    while True:
        # Delete data property to tell owner to give us more data
        w.delete_property(data_atom)

        # Wait for notification that we got data
        while True:
            e = d.next_event()
            if (e.type == X.PropertyNotify
                and e.state == X.PropertyNewValue
                and e.window == w
                and e.atom == data_atom):
                break

        r = w.get_full_property(data_atom, X.AnyPropertyType,
                                sizehint = 10000)

        # End of data
        if len(r.value) == 0:
            return

        output_data(d, r, target_name)
        # loop around


def output_data(d, r, target_name):
    log('got {0}:{1}, length {2}',
        d.get_atom_name(r.property_type),
        r.format,
        len(r.value))

    if r.format == 8:
        if r.property_type == Xatom.STRING:
            value = r.value.decode('ISO-8859-1')
        elif r.property_type == d.get_atom('UTF8_STRING'):
            value = r.value.decode('UTF-8')
        else:
            value = binascii.hexlify(r.value).decode('ascii')
        sys.stdout.write(value)

    elif r.format == 32 and r.property_type == Xatom.ATOM:
        for v in r.value:
            sys.stdout.write('{0}\n'.format(d.get_atom_name(v)))

    else:
        for v in r.value:
            sys.stdout.write('{0}\n'.format(v))



if __name__ == '__main__':
    main()



#!/usr/bin/python
#    Copyright (C) 2011 Arun Balasubramanian <med.diagnostix@gmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
#    Free Software Foundation, Inc.,
#    59 Temple Place,
#    Suite 330,
#    Boston, MA 02111-1307 USA


'''
xlsatoms - list atoms on X server (see man xlsatoms)
sample program using python Xlib that mimics the standard xlsatoms utility
additional capability is to match against regular expressions for atoms

'''

# Python 2/3 compatibility.
from __future__ import print_function

import sys
import os

# Python 2/3 compatibility.
from six import PY2, MAXSIZE

# Change path so we find Xlib
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

import re
from Xlib import display, error
from optparse import OptionParser


if PY2:
    integer_type = long
else:
    integer_type = int

parser = OptionParser()
parser.add_option("-d","--display",dest="display",help="This option specifies the X server to which to connect",metavar="dpy",default=None)
parser.add_option("-n","--name",dest="name",help="This option specifies the name of an atom to list.  If the atom does  not  exist,  a  message  will  be printed on the standard error.",metavar="string",default=None)
parser.add_option("-m","--match",dest="match_re",help="This option specifies a regular expression to match against name of an atom to list.  If the atom does  not  exist,  a  message  will  be printed on the standard error.",metavar="reg-exp",default=None)
parser.add_option("-f","--format",dest="format",help="This  option  specifies a printf-style string used to list each atom <value,name> pair, printed in  that  order  (value  is  an unsigned  long  and  name is a char *).  Xlsatoms will supply a newline at the end of each line.  The default is %ld\\t%s.",metavar="string",default="%ld\t%s")
parser.add_option("-r","--range",dest="range",help="This option specifies the range of atom values  to  check.   If low  is not given, a value of 1 assumed.  If high is not given, xlsatoms will stop at the first undefined atom at or above low.",metavar="[low]-[high]",default="1-")

(options, args) = parser.parse_args()

low = 1
high = 1
ec = error.CatchError(error.BadAtom)
d = display.Display(options.display)

def print_atom(print_format,atom,value):
    print(print_format%(atom,value))

def list_atoms(d,re_obj,low,high):
    while(low <= high):
        try:
            val = d.get_atom_name(low)
            if (re_obj == None) :
                print_atom(options.format,low,val)
            elif re_obj.match(val) != None:
                print_atom(options.format,low,val)
            low += 1
        except:
            sys.exit(0)

if options.name != None:
    try:
        atom = d.intern_atom(options.name)
        val = d.get_atom_name(atom)
        print_atom(options.format,atom,val)
    except:
        sys.stderr.write('xlsatoms:  no atom named "%s" on server "%s"'%(options.name,options.display))
        sys.stderr.write("\n")
        sys.exit(1)
    sys.exit(0)

rangeVals = options.range.split("-")
if rangeVals[0] != "":
    low = integer_type(rangeVals[0])

if rangeVals[1] != "":
    high = integer_type(rangeVals[1])
else:
    high = MAXSIZE

if options.match_re != None:
    re_obj = re.compile(options.match_re)
else:
    re_obj = None

list_atoms(d,re_obj,low,high)



#!/usr/bin/env python
#
# examples/put_selection.py -- demonstrate putting selections
# (equivalent to copying to the clipboard)
#
#	Copyright (C) 2013 Peter Liljenberg <peter.liljenberg@gmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
#    Free Software Foundation, Inc.,
#    59 Temple Place,
#    Suite 330,
#    Boston, MA 02111-1307 USA

import sys
import os

# Change path so we find Xlib
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from Xlib import X, display, Xutil, Xatom
from Xlib.protocol import event


# This script does not implement fancy stuff like INCR or MULTIPLE, so
# put a hard limit on the amount of data we allow sending this way
MAX_SIZE = 50000


def log(msg, *args):
    sys.stderr.write(msg.format(*args) + '\n')

def error(msg, *args):
    log(msg, *args)
    sys.exit(1)


def main():
    if len(sys.argv) < 3:
        sys.exit('usage: {0} SELECTION TYPE [FILE [TYPE [FILE...]]]\n\n'
                 'SELECTION is typically PRIMARY, SECONDARY or CLIPBOARD.\n'
                 'If FILE is omitted, stdin is read.\n'
                 'Multiple type/file combos can be specified.\n'
                 .format(sys.argv[0]))

    args = sys.argv[1:]

    d = display.Display()

    sel_name = args[0]
    del args[0]
    sel_atom = d.get_atom(sel_name)

    # map type atom -> data
    types = {}

    while args:
        type_atom = d.get_atom(args[0])
        del args[0]

        if args:
            f = open(args[0], 'rb')
            del args[0]
        else:
            f = sys.stdin

        # Limit selection size to avoid implementing INCR
        data = f.read(MAX_SIZE)
        f.close()

        if len(data) >= MAX_SIZE:
            log('too much data: {0} bytes, max is {1} bytes', len(data), MAX_SIZE)
            error('INCR support not implemented to put larger files')

        types[type_atom] = data


    targets_atom = d.get_atom('TARGETS')


    # We must have a window to own a selection
    w = d.screen().root.create_window(
        0, 0, 10, 10, 0, X.CopyFromParent)

    # And to grab the selection we must have a timestamp, get one with
    # a property notify when we're anyway setting wm_name
    w.change_attributes(event_mask = X.PropertyChangeMask)
    w.set_wm_name(os.path.basename(sys.argv[0]))

    e = d.next_event()
    sel_time = e.time
    w.change_attributes(event_mask = 0)

    # Grab the selection and make sure we actually got it
    w.set_selection_owner(sel_atom, sel_time)
    if d.get_selection_owner(sel_atom) != w:
        log('could not take ownership of {0}', sel_name)
        return

    log('took ownership of selection {0}', sel_name)

    # The event loop, waiting for and processing requests
    while True:
        e = d.next_event()

        if (e.type == X.SelectionRequest
            and e.owner == w
            and e.selection == sel_atom):

            client = e.requestor

            if e.property == X.NONE:
                log('request from obsolete client!')
                client_prop = e.target # per ICCCM recommendation
            else:
                client_prop = e.property

            target_name = d.get_atom_name(e.target)

            log('got request for {0}, dest {1} on 0x{2:08x} {3}',
                target_name, d.get_atom_name(client_prop),
                client.id, client.get_wm_name())

            # Is the client asking for which types we support?
            if e.target == targets_atom:
                # Then respond with TARGETS and the type
                prop_value = [targets_atom] + types.keys()
                prop_type = Xatom.ATOM
                prop_format = 32

            # Request for the offered type
            elif e.target in types:
                prop_value = types[e.target]
                prop_type = e.target
                prop_format = 8

            # Something else, tell client they can't get it
            else:
                log('refusing conversion to {0}', target_name)
                client_prop = X.NONE

            # Put the data on the dest window, if possible
            if client_prop != X.NONE:
                client.change_property(
                    client_prop, prop_type, prop_format, prop_value)

            # And always send a selection notification
            ev = event.SelectionNotify(
                time = e.time,
                requestor = e.requestor,
                selection = e.selection,
                target = e.target,
                property = client_prop)

            client.send_event(ev)

            # Done!

        elif (e.type == X.SelectionClear
              and e.window == w
              and e.atom == sel_atom):
            log('lost ownership of selection {0}', sel_name)
            return

        # A proper owner would also look for PropertyNotify here on
        # the selector's windows to implement INCR and waiting for
        # acknowledgement that the client has finished copying.


if __name__ == '__main__':
    main()



#!/usr/bin/python3
#
# examples/xfixes-selection-notify.py -- demonstrate the XFIXES extension
# SelectionNotify event.
#
#    Copyright (C) 2019
#      Tony Crisci <tony@dubstepdish.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
#    Free Software Foundation, Inc.,
#    59 Temple Place,
#    Suite 330,
#    Boston, MA 02111-1307 USA

# Python 2/3 compatibility.
from __future__ import print_function

import sys
import os
import time

# Change path so we find Xlib
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from Xlib.display import Display
from Xlib.ext import xfixes

def main(argv):
    if len(sys.argv) != 2:
        sys.exit('usage: {0} SELECTION\n\n'
                 'SELECTION is typically PRIMARY, SECONDARY or CLIPBOARD.\n'
                 .format(sys.argv[0]))

    display = Display()

    sel_name = sys.argv[1]
    sel_atom = display.get_atom(sel_name)

    if not display.has_extension('XFIXES'):
        if display.query_extension('XFIXES') is None:
            print('XFIXES extension not supported', file=sys.stderr)
            return 1

    xfixes_version = display.xfixes_query_version()
    print('Found XFIXES version %s.%s' % (
      xfixes_version.major_version,
      xfixes_version.minor_version,
    ), file=sys.stderr)

    screen = display.screen()

    mask = xfixes.XFixesSetSelectionOwnerNotifyMask | \
           xfixes.XFixesSelectionWindowDestroyNotifyMask | \
           xfixes.XFixesSelectionClientCloseNotifyMask

    display.xfixes_select_selection_input(screen.root, sel_atom, mask)

    while True:
        e = display.next_event()
        print(e)

        if (e.type, e.sub_code) == display.extension_event.SetSelectionOwnerNotify:
            print('SetSelectionOwner: owner=0x{0:08x}'.format(e.owner.id))
        elif (e.type, e.sub_code) == display.extension_event.SelectionWindowDestroyNotify:
            print('SelectionWindowDestroy: owner=0x{0:08x}'.format(e.owner.id))
        elif (e.type, e.sub_code) == display.extension_event.SelectionClientCloseNotify:
            print('SelectionClientClose: owner=0x{0:08x}'.format(e.owner.id))


if __name__ == '__main__':
    sys.exit(main(sys.argv))



#!/usr/bin/python
#
# examples/xres.py -- demonstrate the X-Resource extension
#
#    Copyright (C) 2021 Aleksei Bavshin <alebastr89@gmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
#    Free Software Foundation, Inc.,
#    51 Franklin Street,
#    Fifth Floor,
#    Boston, MA 02110-1301 USA

import os
import sys

# Change path so we find Xlib
sys.path.insert(0, os.path.join(os.path.dirname(__file__), ".."))

from Xlib.display import Display
from Xlib.ext import res as XRes


def check_ext(disp, extname, version):
    if disp.query_extension(extname) is None:
        raise AssertionError("Server has {} extension".format(extname))

    r = disp.res_query_version()
    if (r.server_major, r.server_minor) < version:
        raise AssertionError(
            "Server has requested version {} of {} extension".format(version, extname)
        )


def query_client_id(display, wid):
    specs = [{"client": wid, "mask": XRes.LocalClientPIDMask}]
    r = display.res_query_client_ids(specs)
    for id in r.ids:
        if id.spec.client > 0 and id.spec.mask == XRes.LocalClientPIDMask:
            for value in id.value:
                return value
    return None


def print_client_info(disp, client):
    print("client: {}".format(client))

    resources = disp.res_query_client_resources(client)
    rc = [r.count for r in resources.types]
    print("\tresouces: {} resources of {} types".format(sum(rc), len(rc)))

    pb = disp.res_query_client_pixmap_bytes(client)
    print("\tpixmaps: {} bytes {} overflow".format(pb.bytes, pb.bytes_overflow))

    pid = query_client_id(disp, client)
    print("\tpid: {}".format(pid))

    rb = disp.res_query_resource_bytes(client, [{"resource": 0, "type": 0}])
    sizes = [s.size.bytes for s in rb.sizes]
    print("\t{} resources consume {} bytes".format(len(sizes), sum(sizes)))


def main():
    display = Display()
    check_ext(display, XRes.extname, (1, 2))

    clients = display.res_query_clients().clients
    print("{} clients connected to the server".format(len(clients)))

    for client in clients:
        print_client_info(display, client.resource_base)


if __name__ == "__main__":
    main()



#!/usr/bin/env python2

import sys, os
sys.path.insert(0, os.path.normpath(os.path.join(__file__, '../..')))

import unittest
from Xlib.protocol import request, event
from . import LittleEndianTest as EndianTest
from . import DummyDisplay

dummy_display = DummyDisplay()


class TestKeymapNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'data': [248, 202, 136, 187, 201, 200, 244, 146, 187, 195, 178, 236, 157, 185, 166, 196, 164, 146, 143, 170, 206, 201, 240, 159, 247, 205, 231, 197, 254, 240, 148],
            'type': 138,
            }
        self.evt_bin_0 = b'\x8a\xf8\xca\x88' b'\xbb\xc9\xc8\xf4' \
            b'\x92\xbb\xc3\xb2' b'\xec\x9d\xb9\xa6' \
            b'\xc4\xa4\x92\x8f' b'\xaa\xce\xc9\xf0' \
            b'\x9f\xf7\xcd\xe7' b'\xc5\xfe\xf0\x94'


    def testPack0(self):
        bin = event.KeymapNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.KeymapNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestExpose(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'count': 15258,
            'height': 6241,
            'sequence_number': 48394,
            'type': 238,
            'width': 1951,
            'window': 692245859,
            'x': 39820,
            'y': 16664,
            }
        self.evt_bin_0 = b'\xee\x00\x0a\xbd' b'\x63\xd5\x42\x29' \
            b'\x8c\x9b\x18\x41' b'\x9f\x07\x61\x18' \
            b'\x9a\x3b\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.Expose._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.Expose._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestGraphicsExpose(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'count': 49818,
            'drawable': 1443779242,
            'height': 2892,
            'major_event': 172,
            'minor_event': 50267,
            'sequence_number': 50375,
            'type': 133,
            'width': 38020,
            'x': 54088,
            'y': 17918,
            }
        self.evt_bin_0 = b'\x85\x00\xc7\xc4' b'\xaa\x52\x0e\x56' \
            b'\x48\xd3\xfe\x45' b'\x84\x94\x4c\x0b' \
            b'\x5b\xc4\x9a\xc2' b'\xac\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.GraphicsExpose._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.GraphicsExpose._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestNoExpose(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'major_event': 199,
            'minor_event': 29237,
            'sequence_number': 34266,
            'type': 248,
            'window': 1399171519,
            }
        self.evt_bin_0 = b'\xf8\x00\xda\x85' b'\xbf\xa9\x65\x53' \
            b'\x35\x72\xc7\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.NoExpose._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.NoExpose._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestVisibilityNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'sequence_number': 38616,
            'state': 253,
            'type': 174,
            'window': 936121409,
            }
        self.evt_bin_0 = b'\xae\x00\xd8\x96' b'\x41\x14\xcc\x37' \
            b'\xfd\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.VisibilityNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.VisibilityNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestCreateNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'border_width': 56468,
            'height': 7111,
            'override': 0,
            'parent': 747306217,
            'sequence_number': 31058,
            'type': 151,
            'width': 44173,
            'window': 876986399,
            'x': -21847,
            'y': -22248,
            }
        self.evt_bin_0 = b'\x97\x00\x52\x79' b'\xe9\xfc\x8a\x2c' \
            b'\x1f\xc0\x45\x34' b'\xa9\xaa\x18\xa9' \
            b'\x8d\xac\xc7\x1b' b'\x94\xdc\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.CreateNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.CreateNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestDestroyNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'event': 1489718405,
            'sequence_number': 27233,
            'type': 212,
            'window': 1064077163,
            }
        self.evt_bin_0 = b'\xd4\x00\x61\x6a' b'\x85\x4c\xcb\x58' \
            b'\x6b\x87\x6c\x3f' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.DestroyNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.DestroyNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestUnmapNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'event': 2029853215,
            'from_configure': 0,
            'sequence_number': 43679,
            'type': 201,
            'window': 860030193,
            }
        self.evt_bin_0 = b'\xc9\x00\x9f\xaa' b'\x1f\x1a\xfd\x78' \
            b'\xf1\x04\x43\x33' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.UnmapNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.UnmapNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestMapNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'event': 675485985,
            'override': 1,
            'sequence_number': 6027,
            'type': 244,
            'window': 542087937,
            }
        self.evt_bin_0 = b'\xf4\x00\x8b\x17' b'\x21\x19\x43\x28' \
            b'\x01\x9b\x4f\x20' b'\x01\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.MapNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.MapNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestMapRequest(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'parent': 1659099581,
            'sequence_number': 63838,
            'type': 157,
            'window': 868024861,
            }
        self.evt_bin_0 = b'\x9d\x00\x5e\xf9' b'\xbd\xd9\xe3\x62' \
            b'\x1d\x02\xbd\x33' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.MapRequest._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.MapRequest._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestReparentNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'event': 1867017989,
            'override': 0,
            'parent': 992152190,
            'sequence_number': 43356,
            'type': 128,
            'window': 1165276406,
            'x': -19227,
            'y': -30992,
            }
        self.evt_bin_0 = b'\x80\x00\x5c\xa9' b'\x05\x6f\x48\x6f' \
            b'\xf6\xb4\x74\x45' b'\x7e\x0a\x23\x3b' \
            b'\xe5\xb4\xf0\x86' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.ReparentNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.ReparentNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestConfigureNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'above_sibling': 1343120585,
            'border_width': 17757,
            'event': 1624514845,
            'height': 13596,
            'override': 0,
            'sequence_number': 41060,
            'type': 220,
            'width': 3638,
            'window': 1070571314,
            'x': -18284,
            'y': -7865,
            }
        self.evt_bin_0 = b'\xdc\x00\x64\xa0' b'\x1d\x21\xd4\x60' \
            b'\x32\x9f\xcf\x3f' b'\xc9\x64\x0e\x50' \
            b'\x94\xb8\x47\xe1' b'\x36\x0e\x1c\x35' \
            b'\x5d\x45\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.ConfigureNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.ConfigureNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestConfigureRequest(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'border_width': 52639,
            'height': 40159,
            'parent': 1499546058,
            'sequence_number': 57983,
            'sibling': 260826075,
            'stack_mode': 240,
            'type': 201,
            'value_mask': 15938,
            'width': 41545,
            'window': 1040976198,
            'x': -31823,
            'y': -880,
            }
        self.evt_bin_0 = b'\xc9\xf0\x7f\xe2' b'\xca\x41\x61\x59' \
            b'\x46\x09\x0c\x3e' b'\xdb\xe3\x8b\x0f' \
            b'\xb1\x83\x90\xfc' b'\x49\xa2\xdf\x9c' \
            b'\x9f\xcd\x42\x3e' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.ConfigureRequest._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.ConfigureRequest._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestGravityNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'event': 1182983311,
            'sequence_number': 20621,
            'type': 168,
            'window': 29431224,
            'x': -14672,
            'y': -19399,
            }
        self.evt_bin_0 = b'\xa8\x00\x8d\x50' b'\x8f\xe4\x82\x46' \
            b'\xb8\x15\xc1\x01' b'\xb0\xc6\x39\xb4' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.GravityNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.GravityNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestResizeRequest(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'height': 60085,
            'sequence_number': 14981,
            'type': 151,
            'width': 55398,
            'window': 2130921516,
            }
        self.evt_bin_0 = b'\x97\x00\x85\x3a' b'\x2c\x48\x03\x7f' \
            b'\x66\xd8\xb5\xea' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.ResizeRequest._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.ResizeRequest._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestPropertyNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'atom': 252322161,
            'sequence_number': 30497,
            'state': 167,
            'time': 1391011497,
            'type': 157,
            'window': 2033863003,
            }
        self.evt_bin_0 = b'\x9d\x00\x21\x77' b'\x5b\x49\x3a\x79' \
            b'\x71\x21\x0a\x0f' b'\xa9\x26\xe9\x52' \
            b'\xa7\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.PropertyNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.PropertyNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestSelectionClear(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'atom': 1092859866,
            'sequence_number': 61378,
            'time': 1538959461,
            'type': 255,
            'window': 626833463,
            }
        self.evt_bin_0 = b'\xff\x00\xc2\xef' b'\x65\xa8\xba\x5b' \
            b'\x37\xb8\x5c\x25' b'\xda\xb7\x23\x41' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.SelectionClear._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.SelectionClear._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestSelectionRequest(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'owner': 1036225485,
            'property': 981007010,
            'requestor': 2105124856,
            'selection': 1014149797,
            'sequence_number': 27485,
            'target': 523473665,
            'time': 1792621552,
            'type': 197,
            }
        self.evt_bin_0 = b'\xc5\x00\x5d\x6b' b'\xf0\x3b\xd9\x6a' \
            b'\xcd\x8b\xc3\x3d' b'\xf8\xa7\x79\x7d' \
            b'\xa5\xb2\x72\x3c' b'\x01\x93\x33\x1f' \
            b'\xa2\xfa\x78\x3a' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.SelectionRequest._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.SelectionRequest._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestSelectionNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'property': 1602716574,
            'requestor': 1979762314,
            'selection': 1287219120,
            'sequence_number': 25394,
            'target': 1091504539,
            'time': 409398186,
            'type': 165,
            }
        self.evt_bin_0 = b'\xa5\x00\x32\x63' b'\xaa\xeb\x66\x18' \
            b'\x8a\xc6\x00\x76' b'\xb0\x67\xb9\x4c' \
            b'\x9b\x09\x0f\x41' b'\x9e\x83\x87\x5f' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.SelectionNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.SelectionNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestColormapNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'colormap': 593302316,
            'new': 1,
            'sequence_number': 56880,
            'state': 215,
            'type': 162,
            'window': 149981547,
            }
        self.evt_bin_0 = b'\xa2\x00\x30\xde' b'\x6b\x89\xf0\x08' \
            b'\x2c\x13\x5d\x23' b'\x01\xd7\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.ColormapNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.ColormapNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestClientMessage(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'client_type': 607457628,
            'data': (8, b'01234567890123456789'),
            'sequence_number': 54031,
            'type': 196,
            'window': 5574388,
            }
        self.evt_bin_0 = b'\xc4\x08\x0f\xd3' b'\xf4\x0e\x55\x00' \
            b'\x5c\x11\x35\x24' b'\x30\x31\x32\x33' \
            b'\x34\x35\x36\x37' b'\x38\x39\x30\x31' \
            b'\x32\x33\x34\x35' b'\x36\x37\x38\x39'

        self.evt_args_1 = {
            'client_type': 1245441508,
            'data': (16, [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]),
            'sequence_number': 55140,
            'type': 204,
            'window': 1260504694,
            }
        self.evt_bin_1 = b'\xcc\x10\x64\xd7' b'\x76\xc6\x21\x4b' \
            b'\xe4\xed\x3b\x4a' b'\x01\x00\x02\x00' \
            b'\x03\x00\x04\x00' b'\x05\x00\x06\x00' \
            b'\x07\x00\x08\x00' b'\x09\x00\x0a\x00'

        self.evt_args_2 = {
            'client_type': 959018764,
            'data': (32, [1, 2, 3, 4, 5]),
            'sequence_number': 56961,
            'type': 253,
            'window': 319171761,
            }
        self.evt_bin_2 = b'\xfd\x20\x81\xde' b'\xb1\x2c\x06\x13' \
            b'\x0c\x77\x29\x39' b'\x01\x00\x00\x00' \
            b'\x02\x00\x00\x00' b'\x03\x00\x00\x00' \
            b'\x04\x00\x00\x00' b'\x05\x00\x00\x00'


    def testPack0(self):
        bin = event.ClientMessage._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.ClientMessage._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)

    def testPack1(self):
        bin = event.ClientMessage._fields.to_binary(*(), **self.evt_args_1)
        self.assertBinaryEqual(bin, self.evt_bin_1)

    def testUnpack1(self):
        args, remain = event.ClientMessage._fields.parse_binary(self.evt_bin_1, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_1)

    def testPack2(self):
        bin = event.ClientMessage._fields.to_binary(*(), **self.evt_args_2)
        self.assertBinaryEqual(bin, self.evt_bin_2)

    def testUnpack2(self):
        args, remain = event.ClientMessage._fields.parse_binary(self.evt_bin_2, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_2)


class TestMappingNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'count': 244,
            'first_keycode': 224,
            'request': 213,
            'sequence_number': 22874,
            'type': 251,
            }
        self.evt_bin_0 = b'\xfb\x00\x5a\x59' b'\xd5\xe0\xf4\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.MappingNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.MappingNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

import sys, os
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

import unittest
from Xlib.protocol import rq

class Custom(rq.Card32):
    def check_value(self, value):
        return int(value)
    def parse_value(self, value, display):
        return str(value)

CustomObj = Custom(None)


class TestNestedStruct(unittest.TestCase):
    def setUp(self):
        self.s1 = rq.Struct(rq.Card32('a1'), rq.Card32('a2'))
        self.s2 = rq.Struct(rq.Object('b1', self.s1), rq.Object('b2', self.s1))
        self.s3 = rq.Struct(rq.Object('c1', self.s2), rq.Object('c2', self.s2))

    def testToBinary1(self):
        self.s1.to_binary(1, 2)

    def testToBinary2(self):
        self.s2.to_binary((1, 2), (3, 4))

    def testToBinary3(self):
        self.s3.to_binary(((1, 2), (3, 4)), ((5, 6), (7, 8)))

class TestNestedStructWithFancyValue(unittest.TestCase):
    def setUp(self):
        self.s1 = rq.Struct(Custom('a1'), Custom('a2'))
        self.s2 = rq.Struct(rq.Object('b1', self.s1), rq.Object('b2', self.s1))
        self.s3 = rq.Struct(rq.Object('c1', self.s2), rq.Object('c2', self.s2))

    def testToBinary1(self):
        self.s1.to_binary('1', '2')

    def testToBinary2(self):
        self.s2.to_binary(('1', '2'), ('3', '4'))

    def testToBinary3(self):
        self.s3.to_binary((('1', '2'), ('3', '4')), (('5', '6'), ('7', '8')))

class TestListWithFancyValue(unittest.TestCase):
    def setUp(self):
        self.s1 = rq.Struct(rq.LengthOf('l', 4), rq.List('l', CustomObj))

    def testToBinary1(self):
        self.s1.to_binary(['1', '2', '3'])

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*

from functools import partial
import socket
import sys
import unittest

from mock import patch

from Xlib.support import unix_connect
from Xlib.error import DisplayConnectionError, DisplayNameError


@unittest.skipUnless(sys.platform.startswith('linux'), 'Linux specific tests')
class TestUnixConnect(unittest.TestCase):

    def test_get_display(self):
        # Valid cases.
        for display, expected in (
            # Implicit Unix socket connections.
            (':0.1', (None, '', 0, 1)),
            (':4', (None, '', 4, 0)),
            # Implicit TCP connections.
            ('foo:1.2', (None, 'foo', 1, 2)),
            ('bar:5', (None, 'bar', 5, 0)),
            # Explicit Unix socket connections.
            ('unix/foo:4.3', ('unix', 'foo', 4, 3)),
            ('unix/:66', ('unix', '', 66, 0)),
            # Explicit TCP connections.
            ('tcp/foo:11.1', ('tcp', 'foo', 11, 1)),
            ('tcp/bar:66.6', ('tcp', 'bar', 66, 6)),
            ('tcp/unix:54.3', ('tcp', 'unix', 54, 3)),
            # Special case: `unix:0.0` is equivalent to `:0.0`.
            ('unix:99.5', (None, 'unix', 99, 5)),
            ('unix:42', (None, 'unix', 42, 0)),
        ):
            result = unix_connect.get_display(display)
            self.assertEqual(result, (display,) + expected)
        # Invalid cases.
        for display in (
            # No display number.
            '',
            ':',
            'foo',
            'bar:',
            # Bad screen number.
            ':48.',
            ':47.f',
            # Bad hostname.
            u'fòó:0',
            u'tcp/bàr:1',
            u'unix/fòóbàr:2',
            # Bad protocol.
            'udp/foo:0'
            # With explicit TCP connections, hostname must be set.
            'tcp/:0',
        ):
            with self.assertRaises(DisplayNameError):
                unix_connect.get_display(display)

    def test_get_socket(self):
        class FakeSocket(object):
            def fileno(self):
                return 42
        calls = []
        def _get_socket(socket_type, raises, *params):
            calls.append(('_get_%s_socket' % socket_type,) + params)
            if raises:
                raise socket.error()
            return FakeSocket()
        def path_exists(returns, path):
            calls.append(('os.path.exists', path))
            return returns
        def ensure_not_inheritable(*args):
            calls.append(('ensure_not_inheritable',) + args)
        for params, allow_unix, unix_addr_exists, allow_tcp, expect_connection_error, expected_calls in (
            # Successful explicit TCP socket connection.
            (('tcp/host:6', None, 'host', 6), False, False, True, False, [
                ('_get_tcp_socket', 'host', 6),
            ]),
            # Failed explicit TCP socket connection.
            (('tcp/host:6', None, 'host', 6), False, False, False, True, [
                ('_get_tcp_socket', 'host', 6),
            ]),
            # Successful implicit TCP socket connection.
            (('host:5', None, 'host', 5), False, False, True, False, [
                ('_get_tcp_socket', 'host', 5),
            ]),
            # Failed implicit TCP socket connection.
            (('host:5', None, 'host', 5), False, False, False, True, [
                ('_get_tcp_socket', 'host', 5),
            ]),
            # Successful explicit Unix socket connection.
            (('unix/name:0', 'unix', 'name', 0), True, True, False, False, [
                ('os.path.exists', '/tmp/.X11-unix/X0'),
                ('_get_unix_socket', '/tmp/.X11-unix/X0'),
            ]),
            # Failed explicit Unix socket connection.
            (('unix/name:0', 'unix', 'name', 0), False, True, False, True, [
                ('os.path.exists', '/tmp/.X11-unix/X0'),
                ('_get_unix_socket', '/tmp/.X11-unix/X0'),
            ]),
            # Successful explicit Unix socket connection, variant.
            (('unix:0', None, 'unix', 0), True, True, False, False, [
                ('os.path.exists', '/tmp/.X11-unix/X0'),
                ('_get_unix_socket', '/tmp/.X11-unix/X0'),
            ]),
            # Failed explicit Unix socket connection, variant.
            (('unix:0', None, 'unix', 0), False, True, False, True, [
                ('os.path.exists', '/tmp/.X11-unix/X0'),
                ('_get_unix_socket', '/tmp/.X11-unix/X0'),
            ]),
            # Successful implicit Unix socket connection.
            ((':4', None, '', 4), True, True, False, False, [
                ('os.path.exists', '/tmp/.X11-unix/X4'),
                ('_get_unix_socket', '/tmp/.X11-unix/X4'),
            ]),
            # Successful implicit Unix socket connection, abstract address.
            ((':3', None, '', 3), True, False, False, False, [
                ('os.path.exists', '/tmp/.X11-unix/X3'),
                ('_get_unix_socket', '\0/tmp/.X11-unix/X3'),
            ]),
            # Failed implicit Unix socket connection, successful fallback on TCP.
            ((':2', None, '', 2), False, False, True, False, [
                ('os.path.exists', '/tmp/.X11-unix/X2'),
                ('_get_unix_socket', '\0/tmp/.X11-unix/X2'),
                ('_get_tcp_socket', '', 2),
            ]),
            # Failed implicit Unix socket connection, failed fallback on TCP.
            ((':1', None, '', 1), False, False, False, True, [
                ('os.path.exists', '/tmp/.X11-unix/X1'),
                ('_get_unix_socket', '\0/tmp/.X11-unix/X1'),
                ('_get_tcp_socket', '', 1),
            ]),
        ):
            with \
                    patch('Xlib.support.unix_connect._get_unix_socket',
                       partial(_get_socket, 'unix', not allow_unix)), \
                    patch('Xlib.support.unix_connect._get_tcp_socket',
                          partial(_get_socket, 'tcp', not allow_tcp)), \
                    patch('os.path.exists',
                          partial(path_exists, unix_addr_exists)), \
                    patch('Xlib.support.unix_connect._ensure_not_inheritable', ensure_not_inheritable):
                del calls[:]
                if expect_connection_error:
                    with self.assertRaises(DisplayConnectionError):
                        unix_connect.get_socket(*params)
                else:
                    s = unix_connect.get_socket(*params)
                    self.assertIsInstance(s, FakeSocket)
                    expected_calls.append(('ensure_not_inheritable', s))
                self.assertEqual(calls, expected_calls)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
#
# examples/xfixes.py -- demonstrate the XFIXES extension
#
#    Copyright (C) 2011 Outpost Embedded, LLC
#      Forest Bond <forest.bond@rapidrollout.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
#    Free Software Foundation, Inc.,
#    59 Temple Place,
#    Suite 330,
#    Boston, MA 02111-1307 USA

# Python 2/3 compatibility.
from __future__ import print_function

import sys
import os
import time

# Change path so we find Xlib
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from Xlib.display import Display


def main(argv):
    display = Display()

    if not display.has_extension('XFIXES'):
        if display.query_extension('XFIXES') is None:
            print('XFIXES extension not supported', file=sys.stderr)
            return 1

    xfixes_version = display.xfixes_query_version()
    print('Found XFIXES version %s.%s' % (
      xfixes_version.major_version,
      xfixes_version.minor_version,
    ), file=sys.stderr)

    screen = display.screen()

    print('Hiding cursor ...', file=sys.stderr)
    screen.root.xfixes_hide_cursor()
    display.sync()

    time.sleep(5)

    print('Showing cursor ...', file=sys.stderr)
    screen.root.xfixes_show_cursor()
    display.sync()


if __name__ == '__main__':
    sys.exit(main(sys.argv))




import unittest
import binascii
import difflib
import struct
import array

import Xlib.protocol.event
import Xlib.protocol.rq


class CmpArray(object):

    def __init__(self, *args, **kws):
        self.array = array.array(*args, **kws)

    def __len__(self):
        return len(self.array)

    def __getitem__(self, key):
        if isinstance(key, slice):
            x = key.start
            y = key.stop
            return list(self.array[x:y])
        else:
            return self.array[key]

    def __getattr__(self, attr):
        return getattr(self.array, attr)

    def __eq__(self, other):
        return self.array.tolist() == other

Xlib.protocol.rq.array = CmpArray


class DummyDisplay(object):

    def get_resource_class(self, x):
        return None

    event_classes = Xlib.protocol.event.event_class


class TestCase(unittest.TestCase):

    def assertBinaryEqual(self, bin1, bin2):
        if bin1 != bin2:
            self.fail('binary contents differ:\n' + bindiff(bin1, bin2))

    def assertBinaryEmpty(self, bin):
        if len(bin) != 0:
            self.fail('binary content not empty:\n' + ''.join(tohex(bin)))

class BigEndianTest(TestCase):

    @classmethod
    def setUpClass(cls):
        if struct.unpack('BB', struct.pack('H', 0x0100))[0] != 1:
            raise unittest.SkipTest('big-endian tests, skipping on this system')

class LittleEndianTest(TestCase):

    @classmethod
    def setUpClass(cls):
        if struct.unpack('BB', struct.pack('H', 0x0100))[0] != 0:
            raise unittest.SkipTest('little-endian tests, skipping on this system')


def tohex(bin):
    hex = []
    for i in range(0, len(bin), 16):
        hex.append(str(binascii.hexlify(bin[i:i+16])) + '\n')
    return hex

def bindiff(bin1, bin2):
    hex1 = tohex(bin1)
    hex2 = tohex(bin2)
    return ''.join(difflib.ndiff(hex1, hex2))
